@node XDR Function Reference
@section XDR Function Reference
@cindex XDR function reference
@cindex Function reference, XDR

These routines allow C programmers to describe arbitrary data structures
in a machine-independent fashion.  Data for remote procedure calls are
transmitted using these routines.

@menu
* xdr_array()::
* xdr_bool()::
* xdr_bytes()::
* xdr_char()::
* xdr_destroy()::
* xdr_double()::
* xdr_enum()::
* xdr_float()::
* xdr_free()::
* xdr_getpos()::
* xdr_inline()::
* xdr_int()::
* xdr_long()::
* xdrmem_create()::
* xdr_opaque()::
* xdr_pointer()::
* xdrrec_create()::
* xdrrec_endofrecord()::
* xdrrec_eof()::
* xdrrec_skiprecord()::
* xdr_reference()::
* xdr_setpos()::
* xdr_short()::
* xdrstdio_create()::
* xdr_string()::
* xdr_u_char()::
* xdr_u_int()::
* xdr_u_long()::
* xdr_u_short()::
* xdr_union()::
* xdr_vector()::
* xdr_void()::
* xdr_wrapstring()::
@end menu

@node xdr_array()
@subsection xdr_array()
@findex xdr_array()

@example
bool_t XDRFUN
xdr_array(xdrs, arrp, sizep, maxsize, elsize, elproc)
XDR *xdrs;
char **arrp;
u_int *sizep, maxsize, elsize;
xdrproc_t elproc;
@end example

@noindent A filter primitive that translates between variable-length
arrays and their corresponding external representations. The parameter
@code{arrp} is the address of the pointer to the array, while
@code{sizep} is the address of the element count of the array; this
element count cannot exceed @code{maxsize}.  The parameter @code{elsize}
is the @code{sizeof} each of the array's elements, and @code{elproc} is
an @code{XDR} filter that translates between the array elements' C form,
and their external representation.  This routine returns one if it
succeeds, zero otherwise.

@need 1600
@node xdr_bool()
@subsection xdr_bool()
@findex xdr_bool()

@example
bool_t XDRFUN
xdr_bool(xdrs, bp)
XDR *xdrs;
bool_t *bp;
@end example

@noindent A filter primitive that translates between booleans (C
integers) and their external representations. When encoding data, this
filter produces values of either one or zero.  This routine returns one
if it succeeds, zero otherwise.

@need 2000
@node xdr_bytes()
@subsection xdr_bytes()
@findex xdr_bytes()

@example
bool_t XDRFUN
xdr_bytes(xdrs, sp, sizep, maxsize)
XDR *xdrs;
char **sp;
u_int *sizep, maxsize;
@end example

@noindent A filter primitive that translates between counted byte
strings and their external representations.  The parameter @code{sp} is
the address of the string pointer. The length of the string is located
at address @code{sizep}; strings cannot be longer than @code{maxsize}.
This routine returns one if it succeeds, zero otherwise.

@need 1400
@node xdr_char()
@subsection xdr_char()
@findex xdr_char()

@example
bool_t XDRFUN
xdr_char(xdrs, cp)
XDR *xdrs;
char *cp;
@end example

@noindent A filter primitive that translates between C characters
and their external representations.  This routine returns one if it
succeeds, zero otherwise.  Note: encoded characters are not packed, and
occupy 4 bytes each. For arrays of characters, it is worthwhile to
consider
@ifinfo
@ref{xdr_bytes()},
@end ifinfo
@iftex
@code{xdr_bytes()},
@end iftex
@ifinfo
@ref{xdr_opaque()}
@end ifinfo
@iftex
@code{xdr_opaque()}
@end iftex
or
@ifinfo
@ref{xdr_string()}.
@end ifinfo
@iftex
@code{xdr_string()}.
@end iftex

@need 1600
@node xdr_destroy()
@subsection xdr_destroy()
@findex xdr_destroy()

@example
void
xdr_destroy(xdrs)
XDR *xdrs;
@end example

@noindent A macro that invokes the destroy routine associated with the
@code{XDR} stream, @code{xdrs}.  Destruction usually involves freeing
private data structures associated with the stream.  Using @code{xdrs}
after invoking @code{xdr_destroy()} is undefined.

@need 1400
@node xdr_double()
@subsection xdr_double()
@findex xdr_double()

@example
bool_t XDRFUN
xdr_double(xdrs, dp)
XDR *xdrs;
double *dp;
@end example

@noindent A filter primitive that translates between C
@code{double} precision numbers and their external representations.
This routine returns one if it succeeds, zero otherwise.

@need 1400
@node xdr_enum()
@subsection xdr_enum()
@findex xdr_enum()

@example
bool_t XDRFUN
xdr_enum(xdrs, ep)
XDR *xdrs;
enum_t *ep;
@end example

@noindent A filter primitive that translates between C
@code{enum}s (actually integers) and their external representations.
This routine returns one if it succeeds, zero otherwise.

@need 1600
@node xdr_float()
@subsection xdr_float()
@findex xdr_float()

@example
bool_t XDRFUN
xdr_float(xdrs, fp)
XDR *xdrs;
float *fp;
@end example

@noindent A filter primitive that translates between C
@code{float}s and their external representations.  This routine returns
one if it succeeds, zero otherwise.

@need 1800
@node xdr_free()
@subsection xdr_free()
@findex xdr_free()

@example
void
xdr_free(proc, objp)
xdrproc_t proc;
char *objp;
@end example

@noindent Generic freeing routine. The first argument is the
@code{XDR} routine for the object being freed. The second argument is a
pointer to the object itself. Note: the pointer passed to this routine
is @code{not} freed, but what it points to @code{is} freed
(recursively).

@need 1600
@node xdr_getpos()
@subsection xdr_getpos()
@findex xdr_getpos()

@example
u_int
xdr_getpos(xdrs)
XDR *xdrs;
@end example

@noindent A macro that invokes the get-position routine
associated with the @code{XDR} stream, @code{xdrs}.  The routine returns
an unsigned integer, which indicates the position of the @code{XDR} byte
stream.  A desirable feature of @code{XDR} streams is that simple
arithmetic works with this number, although the @code{XDR} stream
instances need not guarantee this.

@need 1200
@node xdr_inline()
@subsection xdr_inline()
@findex xdr_inline()

@example
long *
xdr_inline(xdrs, len)
XDR *xdrs;
int len;
@end example

@noindent A macro that invokes the in-line routine associated with the
@code{XDR} stream, @code{xdrs}.  The routine returns a pointer to a
contiguous piece of the stream's buffer; @code{len} is the byte length
of the desired buffer.  Note: pointer is cast to @code{long *}.
@strong{Warning:} @code{xdr_inline()} may return @code{NULL} (0) if it
cannot allocate a contiguous piece of a buffer.  Therefore the behavior
may vary among stream instances; it exists for the sake of efficiency.

@need 1400
@node xdr_int()
@subsection xdr_int()
@findex xdr_int()

@example
bool_t XDRFUN
xdr_int(xdrs, ip)
XDR *xdrs;
int *ip;
@end example

@noindent A filter primitive that translates between C integers
and their external representations.  This routine returns one if it
succeeds, zero otherwise.

@need 1400
@node xdr_long()
@subsection xdr_long()
@findex xdr_long()

@example
bool_t XDRFUN
xdr_long(xdrs, lp)
XDR *xdrs;
long *lp;
@end example

@noindent A filter primitive that translates between C
@code{long} integers and their external representations.  This routine
returns one if it succeeds, zero otherwise.

@need 2400
@node xdrmem_create()
@subsection xdrmem_create()
@findex xdrmem_create()

@example
void
xdrmem_create(xdrs, addr, size, op)
XDR *xdrs;
char *addr;
u_int size;
enum xdr_op op;
@end example

@noindent This routine initializes the 
@code{XDR} stream object pointed to by @code{xdrs}.  The stream's data
is written to, or read from, a chunk of memory at location @code{addr}
whose length is no more than @code{size} bytes long.  The @code{op}
determines the direction of the @code{XDR} stream (either
@code{XDR_ENCODE}, @code{XDR_DECODE}, or @code{XDR_FREE}).

@need 2000
@node xdr_opaque()
@subsection xdr_opaque()
@findex xdr_opaque()

@example
bool_t XDRFUN
xdr_opaque(xdrs, cp, cnt)
XDR *xdrs;
char *cp;
u_int cnt;
@end example

@noindent A filter primitive that translates between fixed size opaque
data and its external representation.  The parameter @code{cp} is the
address of the opaque object, and @code{cnt} is its size in bytes.  This
routine returns one if it succeeds, zero otherwise.

@need 2000
@node xdr_pointer()
@subsection xdr_pointer()
@findex xdr_pointer()

@example
bool_t XDRFUN
xdr_pointer(xdrs, objpp, objsize, xdrobj)
XDR *xdrs;
char **objpp;
u_int objsize;
xdrproc_t xdrobj;
@end example

@noindent Like
@ifinfo
@ref{xdr_reference()}
@end ifinfo
@iftex
@code{xdr_reference()}
@end iftex
execpt that it serializes @code{NULL} pointers, whereas
@ifinfo
@ref{xdr_reference()}
@end ifinfo
@iftex
@code{xdr_reference()}
@end iftex
does not.  Thus, @code{xdr_pointer()} can represent recursive data
structures, such as binary trees or linked lists.

@need 3000
@node xdrrec_create()
@subsection xdrrec_create()
@findex xdrrec_create()

@example
void
xdrrec_create(xdrs, sendsize, recvsize, handle, readit, writeit)
XDR *xdrs;
u_int sendsize, recvsize;
char *handle;
int (*readit) (), (*writeit) ();
@end example

@noindent This routine initializes the @code{XDR} stream object pointed
to by @code{xdrs}.  The stream's data is written to a buffer of size
@code{sendsize}; a value of zero indicates the system should use a
suitable default. The stream's data is read from a buffer of size
@code{recvsize}; it too can be set to a suitable default by passing a
zero value.  When a stream's output buffer is full, @code{writeit} is
called.  Similarly, when a stream's input buffer is empty, @code{readit}
is called.  The behavior of these two routines is similar to the system
calls @code{read} and @code{write}, except that @code{handle} is passed
to the former routines as the first parameter.  Note: the @code{XDR}
stream's @code{op} field must be set by the caller.  Warning: this
@code{XDR} stream implements an intermediate record stream.  Therefore
there are additional bytes in the stream to provide record boundary
information.

@need 1800
@node xdrrec_endofrecord()
@subsection xdrrec_endofrecord()
@findex xdrrec_endofrecord()

@example
bool_t
xdrrec_endofrecord(xdrs, sendnow)
XDR *xdrs;
bool_t sendnow;
@end example

@noindent This routine can be invoked only on streams created by
@ifinfo
@ref{xdrrec_create()}.
@end ifinfo
@iftex
@code{xdrrec_create()}.
@end iftex
The data in the output buffer is marked as a completed record, and the
output buffer is optionally written out if @code{sendnow} is
non-zero. This routine returns one if it succeeds, zero otherwise.

@need 1600
@node xdrrec_eof()
@subsection xdrrec_eof()
@findex xdrrec_eof()

@example
bool_t
xdrrec_eof(xdrs)
XDR *xdrs;
@end example

@noindent This routine can be invoked only on streams created by
@ifinfo
@ref{xdrrec_create()}.
@end ifinfo
@iftex
@code{xdrrec_create()}.
@end iftex
After consuming the rest of the current record in the stream, this
routine returns one if the stream has no more input, zero otherwise.

@need 1000
@node xdrrec_skiprecord()
@subsection xdrrec_skiprecord()
@findex xdrrec_skiprecord()

@example
bool_t
xdrrec_skiprecord(xdrs)
XDR *xdrs;
@end example

@noindent This routine can be invoked only on streams created by
@ifinfo
@ref{xdrrec_create()}.
@end ifinfo
@iftex
@code{xdrrec_create()}.
@end iftex
It tells the @code{XDR} implementation that the rest of the current
record in the stream's input buffer should be discarded.  This routine
returns one if it succeeds, zero otherwise.

@need 2200
@node xdr_reference()
@subsection xdr_reference()
@findex xdr_reference()

@example
bool_t XDRFUN
xdr_reference(xdrs, pp, size, proc)
XDR *xdrs;
char **pp;
u_int size;
xdrproc_t proc;
@end example

@noindent A primitive that provides pointer chasing within structures.
The parameter @code{pp} is the address of the pointer; @code{size} is
the @code{sizeof} the structure that @code{*pp} points to; and
@code{proc} is an @code{XDR} procedure that filters the structure
between its C form and its external representation.  This routine
returns one if it succeeds, zero otherwise.  Warning: this routine does
not understand @code{NULL} pointers. Use
@ifinfo
@ref{xdr_pointer()}
@end ifinfo
@iftex
@code{xdr_pointer()}
@end iftex
instead.

@need 2000
@node xdr_setpos()
@subsection xdr_setpos()
@findex xdr_setpos()

@example
bool_t XDRFUN
xdr_setpos(xdrs, pos)
XDR *xdrs;
u_int pos;
@end example

@noindent A macro that invokes the set position routine associated with
the @code{XDR} stream @code{xdrs}.  The parameter @code{pos} is a
position value obtained from
@ifinfo
@ref{xdr_getpos()}.
@end ifinfo
@iftex
@code{xdr_getpos()}.
@end iftex
This routine returns one if the @code{XDR} stream could be repositioned,
and zero otherwise.  Warning: it is difficult to reposition some types
of @code{XDR} streams, so this routine may fail with one type of stream
and succeed with another.

@need 1600
@node xdr_short()
@subsection xdr_short()
@findex xdr_short()

@example
bool_t XDRFUN
xdr_short(xdrs, sp)
XDR *xdrs;
short *sp;
@end example

@noindent A filter primitive that translates between C @code{short} 
integers and their external representations.  This routine returns one
if it succeeds, zero otherwise.

@need 2000
@node xdrstdio_create()
@subsection xdrstdio_create()
@findex xdrstdio_create()

@example
void
xdrstdio_create(xdrs, file, op)
XDR *xdrs;
FILE *file;
enum xdr_op op;
@end example

@noindent This routine initializes the @code{XDR} stream object pointed
to by @code{xdrs}.  The @code{XDR} stream data is written to, or read
from, the Standard @code{I/O} stream @code{file}.  The parameter
@code{op} determines the direction of the @code{XDR} stream (either
@code{XDR_ENCODE}, @code{XDR_DECODE}, or @code{XDR_FREE}).
@strong{Warning:} the destroy routine associated with such @code{XDR}
streams calls @code{fflush()} on the @code{file} stream, but never
@code{fclose()}.

@need 1800
@node xdr_string()
@subsection xdr_string()
@findex xdr_string()

@example
bool_t XDRFUN
xdr_string(xdrs, sp, maxsize)
XDR *xdrs;
char **sp;
u_int maxsize;
@end example

@noindent A filter primitive that translates between C strings and
their corresponding external representations.  Strings cannot be longer
than @code{maxsize}.  Note: @code{sp} is the address of the string's
pointer.  This routine returns one if it succeeds, zero otherwise.

@need 1600
@node xdr_u_char()
@subsection xdr_u_char()
@findex xdr_u_char()

@example
bool_t XDRFUN
xdr_u_char(xdrs, ucp)
XDR *xdrs;
unsigned char *ucp;
@end example

@noindent A filter primitive that translates between @code{unsigned}
C characters and their external representations.  This routine returns
one if it succeeds, zero otherwise.

@need 1800
@node xdr_u_int()
@subsection xdr_u_int()
@findex xdr_u_int()

@example
bool_t XDRFUN
xdr_u_int(xdrs, up)
XDR *xdrs;
unsigned *up;
@end example

@noindent A filter primitive that translates between C @code{unsigned}
integers and their external representations.  This routine returns one
if it succeeds, zero otherwise.

@need 1400
@node xdr_u_long()
@subsection xdr_u_long()
@findex xdr_u_long()

@example
bool_t XDRFUN
xdr_u_long(xdrs, ulp)
XDR *xdrs;
unsigned long *ulp;
@end example

@noindent A filter primitive that translates between C @code{unsigned long}
integers and their external representations.  This routine returns one
if it succeeds, zero otherwise.

@need 1400
@node xdr_u_short()
@subsection xdr_u_short()
@findex xdr_u_short()

@example
bool_t XDRFUN
xdr_u_short(xdrs, usp)
XDR *xdrs;
unsigned short *usp;
@end example

@noindent A filter primitive that translates between C @code{unsigned short}
integers and their external representations.  This routine returns one
if it succeeds, zero otherwise.

@need 3200
@node xdr_union()
@subsection xdr_union()
@findex xdr_union()

@example
bool_t XDRFUN
xdr_union(xdrs, dscmp, unp, choices, dfault)
XDR *xdrs;
int *dscmp;
char *unp;
struct xdr_discrim *choices;
bool_t (*defaultarm) ();  /* may equal NULL */
@end example

@noindent A filter primitive that translates between a discriminated C
@code{union} and its corresponding external representation. It first
translates the discriminant of the union located at @code{dscmp}.  This
discriminant is always an @code{enum_t}.  Next the union located at
@code{unp} is translated.  The parameter @code{choices} is a pointer to
an array of @code{xdr_discrim} structures. Each structure contains an
ordered pair of [@code{value},@code{proc}].  If the union's discriminant
is equal to the associated @code{value}, then the @code{proc} is called
to translate the union.  The end of the @code{xdr_discrim} structure
array is denoted by a routine of value @code{NULL}.  If the discriminant
is not found in the @code{choices} array, then the @code{defaultarm}
procedure is called (if it is not @code{NULL}).  Returns one if it
succeeds, zero otherwise.

@need 2000
@node xdr_vector()
@subsection xdr_vector()
@findex xdr_vector()

@example
bool_t XDRFUN
xdr_vector(xdrs, arrp, size, elsize, elproc)
XDR *xdrs;
char *arrp;
u_int size, elsize;
xdrproc_t elproc;
@end example

@noindent A filter primitive that translates between fixed-length
arrays and their corresponding external representations.  The parameter
@code{arrp} is the address of the pointer to the array, while
@code{size} is is the element count of the array.  The parameter
@code{elsize} is the @code{sizeof} each of the array's elements, and
@code{elproc} is an @code{XDR} filter that translates between the array
elements' C form, and their external representation.  This routine
returns one if it succeeds, zero otherwise.

@need 1000
@node xdr_void()
@subsection xdr_void()
@findex xdr_void()

@example
bool_t XDRFUN
xdr_void(xdrs, dummy)
XDR *xdrs;
void *dummy;
@end example

@noindent This routine always returns one. It may be passed to
@code{RPC} routines that require a function parameter, where nothing is
to be done. @code{NULL} can be passed as argument pointer.

@need 2000
@node xdr_wrapstring()
@subsection xdr_wrapstring()
@findex xdr_wrapstring()

@example
bool_t XDRFUN
xdr_wrapstring(xdrs, sp)
XDR *xdrs;
char **sp;
@end example

@noindent A primitive that calls @code{xdr_string(xdrs, sp, UINT_MAX);}
where @code{UINT_MAX} is the maximum value of an unsigned integer.
@code{xdr_wrapstring()} is handy because the @code{RPC} package passes a
maximum of two parameters to @code{XDR} routines, and
@ifinfo
@ref{xdr_string()},
@end ifinfo
@iftex
@code{xdr_string()},
@end iftex
one of the most frequently used primitives, requires three.  Returns one
if it succeeds, zero otherwise.

